@@ -25,8 +25,8 @@ ITER <- 400
 threshold <- 0.21
 REFRESH <- 0
 
-source(file.path("helpers", "expect_stanreg.R"))
-source(file.path("helpers", "SW.R"))
+source(test_path("helpers", "expect_stanreg.R"))
+source(test_path("helpers", "SW.R"))
 
 SW(fit <- stan_lm(mpg ~ ., data = mtcars, prior = R2(location = 0.75), 
                   chains = CHAINS, iter = ITER, seed = SEED, refresh = REFRESH))
@@ -131,7 +131,7 @@ test_that("stan_biglm returns stanfit (not stanreg) object ", {
 })
 
 test_that("loo/waic for stan_lm works", {
-  source(file.path("helpers", "expect_equivalent_loo.R"))
+  source(test_path("helpers", "expect_equivalent_loo.R"))
   ll_fun <- rstanarm:::ll_fun
   expect_equivalent_loo(fit)
   expect_identical(ll_fun(fit), rstanarm:::.ll_gaussian_i)
@@ -139,8 +139,8 @@ test_that("loo/waic for stan_lm works", {
 
 context("posterior_predict (stan_lm)")
 test_that("posterior_predict compatible with stan_lm", {
-  source(file.path("helpers", "check_for_error.R"))
-  source(file.path("helpers", "expect_linpred_equal.R"))
+  source(test_path("helpers", "check_for_error.R"))
+  source(test_path("helpers", "expect_linpred_equal.R"))
   
   check_for_error(fit)
   expect_linpred_equal(fit)
